@@ -24,6 +24,7 @@
 #include <gio/gio.h>
 
 #include "dex-block-private.h"
+#include "dex-fd-private.h"
 #include "dex-error.h"
 #include "dex-future-private.h"
 #include "dex-future-set-private.h"
@@ -389,6 +390,7 @@ dex_future_discard (DexFuture *future,
 {
   gboolean has_awaiting = FALSE;
   gboolean matched = FALSE;
+  GQueue discarded = G_QUEUE_INIT;
 
   g_return_if_fail (DEX_IS_FUTURE (future));
   g_return_if_fail (DEX_IS_FUTURE (chained));
@@ -426,12 +428,20 @@ dex_future_discard (DexFuture *future,
           has_awaiting |= cf->awaiting;
 
           g_queue_unlink (&future->chained, &cf->link);
-          dex_chained_future_free (cf);
+          g_queue_push_tail_link (&discarded, &cf->link);
         }
     }
 
   dex_object_unlock (future);
 
+  /* Release chained futures outside of the @future lock */
+  while (discarded.head != NULL)
+    {
+      DexChainedFuture *cf = discarded.head->data;
+      g_queue_unlink (&discarded, &cf->link);
+      dex_chained_future_free (cf);
+    }
+
   /* If we discarded the chained future and there are no more futures
    * awaiting our response, then request the class discard the future,
    * possibly cancelling anything in flight.
@@ -450,7 +460,7 @@ dex_future_discard (DexFuture *future,
 /**
  * dex_future_then: (constructor)
  * @future: (transfer full): a #DexFuture
- * @callback: (scope async): a callback to execute
+ * @callback: (scope notified): a callback to execute
  * @callback_data: closure data for @callback
  * @callback_data_destroy: destroy notify for @callback_data
  *
@@ -480,7 +490,7 @@ DexFuture *
 /**
  * dex_future_then_loop: (constructor)
  * @future: (transfer full): a #DexFuture
- * @callback: (scope async): a callback to execute
+ * @callback: (scope notified) (closure callback_data) (destroy callback_data_destroy): a callback to execute
  * @callback_data: closure data for @callback
  * @callback_data_destroy: destroy notify for @callback_data
  *
@@ -512,7 +522,7 @@ DexFuture *
 /**
  * dex_future_catch_loop: (constructor)
  * @future: (transfer full): a #DexFuture
- * @callback: (scope async): a callback to execute
+ * @callback: (scope notified) (closure callback_data) (destroy callback_data_destroy): a callback to execute
  * @callback_data: closure data for @callback
  * @callback_data_destroy: destroy notify for @callback_data
  *
@@ -544,7 +554,7 @@ DexFuture *
 /**
  * dex_future_finally_loop: (constructor)
  * @future: (transfer full): a #DexFuture
- * @callback: (scope async): a callback to execute
+ * @callback: (scope notified) (closure callback_data) (destroy callback_data_destroy): a callback to execute
  * @callback_data: closure data for @callback
  * @callback_data_destroy: destroy notify for @callback_data
  *
@@ -576,7 +586,7 @@ DexFuture *
 /**
  * dex_future_catch: (constructor)
  * @future: (transfer full): a #DexFuture
- * @callback: (scope async): a callback to execute
+ * @callback: (scope notified) (closure callback_data) (destroy callback_data_destroy): a callback to execute
  * @callback_data: closure data for @callback
  * @callback_data_destroy: destroy notify for @callback_data
  *
@@ -606,7 +616,7 @@ DexFuture *
 /**
  * dex_future_finally: (constructor)
  * @future: (transfer full): a #DexFuture
- * @callback: (scope async): a callback to execute
+ * @callback: (scope notified) (closure callback_data) (destroy callback_data_destroy): a callback to execute
  * @callback_data: closure data for @callback
  * @callback_data_destroy: destroy notify for @callback_data
  *
@@ -922,6 +932,23 @@ DexFuture *
   return dex_ref (static_booleans[!!v_bool]);
 }
 
+/**
+ * dex_future_new_for_fd: (constructor)
+ * @fd: the file-descriptor to take ownership of
+ *
+ * Creates a new future that resolves to @fd.
+ *
+ * This function takes ownership of @fd.
+ *
+ * Returns: (transfer full): a resolved #DexFuture
+ */
+DexFuture *
+(dex_future_new_for_fd) (int fd)
+{
+  GValue gvalue = {DEX_TYPE_FD, {{.v_pointer = g_memdup2 (&fd, sizeof fd)}, {.v_int = 0}}};
+  return dex_static_future_new_steal (&gvalue);
+}
+
 /**
  * dex_future_new_for_int: (constructor)
  * @v_int: the resolved value for the future
@@ -1126,7 +1153,7 @@ DexFuture *
 
 /**
  * dex_future_new_take_variant: (constructor) (skip)
- * @v_variant: the variant to take ownership of
+ * @v_variant: (transfer full): the variant to take ownership of
  *
  * Creates a new #DexFuture that is resolved with @v_variant.
  *
@@ -1339,6 +1366,35 @@ dex_await_pointer (DexFuture  *future,
   return ret;
 }
 
+/**
+ * dex_await_fd: (method)
+ * @future: (transfer full): a #DexFuture
+ * @error: a location for a #GError
+ *
+ * Awaits on @future and returns the resultint file-descriptor.
+ *
+ * The resolved value must be of type %DEX_TYPE_FD or @error is set.
+ *
+ * Returns: a valid file descriptor or -1. you may get -1 without
+ *   error being set if there was no rejected future.
+ */
+int
+dex_await_fd (DexFuture  *future,
+              GError    **error)
+{
+  const GValue *value;
+  int ret = -1;
+
+  g_return_val_if_fail (DEX_IS_FUTURE (future), 0);
+
+  if ((value = dex_await_check (future, DEX_TYPE_FD, error)))
+    ret = dex_fd_steal (g_value_get_boxed (value));
+
+  dex_unref (future);
+
+  return ret;
+}
+
 /**
  * dex_await_int: (method)
  * @future: (transfer full): a #DexFuture
@@ -1374,7 +1430,7 @@ dex_await_int (DexFuture  *future,
  *
  * Awaits on @future and returns the result as an uint.
  *
- * The resolved value must be of type %G_TYPE_INT or @error is set.
+ * The resolved value must be of type %G_TYPE_UINT or @error is set.
  *
  * Returns: an uint, or 0 in case of failure and @error is set.
  */
@@ -1430,7 +1486,7 @@ dex_await_int64 (DexFuture  *future,
  *
  * Awaits on @future and returns the result as an uint64.
  *
- * The resolved value must be of type %G_TYPE_INT64 or @error is set.
+ * The resolved value must be of type %G_TYPE_UINT64 or @error is set.
  *
  * Returns: an uint64, or 0 in case of failure and @error is set.
  */
@@ -1458,7 +1514,7 @@ dex_await_uint64 (DexFuture  *future,
  *
  * Awaits on @future and returns the result as an double.
  *
- * The resolved value must be of type %G_TYPE_INT or @error is set.
+ * The resolved value must be of type %G_TYPE_DOUBLE or @error is set.
  *
  * Returns: an double, or 0 in case of failure and @error is set.
  */
@@ -1486,7 +1542,7 @@ dex_await_double (DexFuture  *future,
  *
  * Awaits on @future and returns the result as an float.
  *
- * The resolved value must be of type %G_TYPE_INT or @error is set.
+ * The resolved value must be of type %G_TYPE_FLOAT or @error is set.
  *
  * Returns: an float, or 0 in case of failure and @error is set.
  */
