@@ -21,6 +21,9 @@
 
 #include "config.h"
 
+#include <unistd.h>
+
+#include "dex-fd-private.h"
 #include "dex-future-private.h"
 #include "dex-promise.h"
 
@@ -167,6 +170,30 @@ dex_promise_reject (DexPromise *promise,
   dex_future_complete (DEX_FUTURE (promise), NULL, error);
 }
 
+/**
+ * dex_promise_resolve_fd:
+ * @promise: a #DexPromise
+ * @fd: a file-descriptor for the resolve to resolve to
+ *
+ * Resolves the promise to @fd.
+ *
+ * The file-descriptor may be dup()'d by this function and
+ * @fd closed immediately.
+ *
+ * Use dex_await_fd() or similar to retrieve the resolved FD.
+ */
+void
+dex_promise_resolve_fd (DexPromise *promise,
+                        int         fd)
+{
+  GValue gvalue = {DEX_TYPE_FD, {{.v_pointer = &fd}, {.v_int = 0}}};
+
+  dex_promise_resolve (promise, &gvalue);
+
+  if (fd > -1)
+    close (fd);
+}
+
 void
 dex_promise_resolve_int (DexPromise *promise,
                          int         value)
@@ -257,7 +284,7 @@ dex_promise_resolve_string (DexPromise *promise,
 /**
  * dex_promise_resolve_object:
  * @promise: a #DexPromise
- * @object: (transfer full) (nullable): a #GObject
+ * @object: (type GObject) (transfer full) (nullable): a #GObject
  *
  */
 void
@@ -275,7 +302,7 @@ dex_promise_resolve_object (DexPromise *promise,
  * @promise: a #DexPromise
  * @variant: (transfer full) (nullable): a #GVariant
  *
- * If @variant is floating, it's reference is consumed.
+ * If @variant is floating, its reference is consumed.
  *
  * Since: 0.8
  */
